@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2018, Christopher C. Hulbert
+ * Copyright (c) 2008-2019, Christopher C. Hulbert
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -210,11 +210,11 @@ EXTERN size_t InflateSkip2(mat_t *mat, matvar_t *matvar, int nbytes);
 EXTERN size_t InflateSkipData(mat_t *mat,z_streamp z,enum matio_types data_type,int len);
 EXTERN size_t InflateVarTag(mat_t *mat, matvar_t *matvar, void *buf);
 EXTERN size_t InflateArrayFlags(mat_t *mat, matvar_t *matvar, void *buf);
-EXTERN size_t InflateDimensions(mat_t *mat, matvar_t *matvar, void *buf);
+EXTERN size_t InflateRankDims(mat_t *mat, matvar_t *matvar, void *buf, size_t nbytes, mat_uint32_t** dims);
 EXTERN size_t InflateVarNameTag(mat_t *mat, matvar_t *matvar, void *buf);
 EXTERN size_t InflateVarName(mat_t *mat,matvar_t *matvar,void *buf,int N);
 EXTERN size_t InflateDataTag(mat_t *mat, matvar_t *matvar, void *buf);
-EXTERN size_t InflateDataType(mat_t *mat, z_stream *matvar, void *buf);
+EXTERN size_t InflateDataType(mat_t *mat, z_stream *z, void *buf);
 EXTERN size_t InflateData(mat_t *mat, z_streamp z, void *buf, unsigned int nBytes);
 EXTERN size_t InflateFieldNameLength(mat_t *mat,matvar_t *matvar,void *buf);
 EXTERN size_t InflateFieldNamesTag(mat_t *mat,matvar_t *matvar,void *buf);
@@ -225,5 +225,8 @@ EXTERN size_t InflateFieldNames(mat_t *mat,matvar_t *matvar,void *buf,int nfield
 /* mat.c */
 EXTERN mat_complex_split_t *ComplexMalloc(size_t nbytes);
 EXTERN enum matio_types ClassType2DataType(enum matio_classes class_type);
+EXTERN int SafeMulDims(const matvar_t *matvar, size_t* nelems);
+EXTERN int SafeMul(size_t* res, size_t a, size_t b);
+EXTERN int SafeAdd(size_t* res, size_t a, size_t b);
 
 #endif
