@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2018, Christopher C. Hulbert
+ * Copyright (c) 2012-2019, Christopher C. Hulbert
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -228,16 +228,16 @@
 #define PACKAGE_NAME "MATIO"
 
 /* Define to the full name and version of this package. */
-#define PACKAGE_STRING "MATIO 1.5.13"
+#define PACKAGE_STRING "MATIO 1.5.16"
 
 /* Define to the one symbol short name of this package. */
 #define PACKAGE_TARNAME "matio"
 
 /* Define to the home page for this package. */
-#define PACKAGE_URL "http://sourceforge.net/projects/matio"
+#define PACKAGE_URL "https://sourceforge.net/projects/matio"
 
 /* Define to the version of this package. */
-#define PACKAGE_VERSION "1.5.13"
+#define PACKAGE_VERSION "1.5.16"
 
 /* The size of `char', as computed by sizeof. */
 #define SIZEOF_CHAR 1
@@ -276,7 +276,7 @@
 #undef STDC_HEADERS
 
 /* Version number of package */
-#define VERSION "1.5.13"
+#define VERSION "1.5.16"
 
 /* Z prefix */
 #undef Z_PREFIX
